@@ -4,8 +4,8 @@ commands and modules.
 '''
 
 import os
-import imp
 import sys
+import importlib
 import traceback
 
 __all__ = ['loadExtensions', 'windows', 'i386', 'darwin', 'amd64', 'gdbstub', 'arm', 'android', 'winkern']
@@ -30,7 +30,7 @@ def loadExtensions(vdb, trace):
     if extdir is None:
         extdir = os.path.abspath(os.path.join('vdb', 'ext'))
 
-    for dirname in extdir.split(';'):
+    for dirname in extdir.split(os.pathsep):
 
         if not os.path.isdir(dirname):
             vdb.vprint('Invalid VDB_EXT_PATH dir: %s' % dirname)
@@ -41,22 +41,24 @@ def loadExtensions(vdb, trace):
             sys.path.append(dirname)
 
         for fname in os.listdir(dirname):
+            modpath = os.path.join(dirname, fname)
+            if os.path.isdir(modpath):
+                modpath = os.path.join(modpath, '__init__.py')
+                if not os.path.exists(modpath()):
+                    continue
+
             if not fname.endswith('.py') or fname == '__init__.py':
                 continue
 
-            modname = os.path.splitext(fname)[0]
-
-            # Build code objects from the module files
-            mod = imp.new_module('vdb.ext.%s' % modname)
-            sys.modules['vdb.ext.%s' % modname] = mod
-            filepath = os.path.join(dirname, fname)
-            with open(filepath, 'r') as fd:
-                filebytes = fd.read()
-                mod.__file__ = filepath
-                try:
-                    exec(filebytes, mod.__dict__)
-                    mod.vdbExtension(vdb, trace)
-                    vdb.addExtension(filepath, mod)
-                except Exception:
-                    vdb.vprint(traceback.format_exc())
-                    vdb.vprint('Extension Error: %s' % filepath)
+            try:
+                # Build code objects from the module files
+                spec = importlib.util.spec_from_file_location(fname, modpath)
+                module = importlib.util.module_from_spec(spec)
+                module.vdb = vdb
+                module.__file__ = modpath
+                spec.loader.exec_module(module)
+                module.vdbExtension(vdb, trace)
+                vdb.addExtension(fname, module)
+            except Exception:
+                vdb.vprint('VDB Extension Error: %s' % modpath)
+                vdb.vprint(traceback.format_exc())
