@@ -1,4 +1,4 @@
-import os
+import binascii
 
 import vivisect.parsers as viv_parsers
 import vstruct.defs.macho as vs_macho
@@ -22,6 +22,7 @@ archcalls = {
     'thumb16': 'armcall',
 }
 
+
 def _loadMacho(vw, filebytes, filename=None, baseaddr=None):
 
     # We fake them to *much* higher than norm so pointer tests do better...
@@ -33,7 +34,7 @@ def _loadMacho(vw, filebytes, filename=None, baseaddr=None):
 
     # grab md5 and sha256 hashes before we modify the bytes
     fhash = viv_parsers.md5Bytes(filebytes)
-    sha256 = v_parsers.sha256Bytes(filebytes)
+    sha256 = viv_parsers.sha256Bytes(filebytes)
 
     # Check for the FAT binary magic...
     if binascii.hexlify(filebytes[:4]) in ('cafebabe', 'bebafeca'):
@@ -53,7 +54,7 @@ def _loadMacho(vw, filebytes, filename=None, baseaddr=None):
             if archname == fatarch:
                 archhdr = ar
                 break
-            archlist.append((archname,ar))
+            archlist.append((archname, ar))
 
         if not archhdr:
             # If we don't have a specified arch, exception!
@@ -77,7 +78,7 @@ def _loadMacho(vw, filebytes, filename=None, baseaddr=None):
     vw.setMeta("Platform", "Darwin")
     vw.setMeta("Format", "macho")
 
-    vw.setMeta('DefaultCall', archcalls.get(arch,'unknown'))
+    vw.setMeta('DefaultCall', archcalls.get(arch, 'unknown'))
 
     # Add the file entry
 
@@ -99,6 +100,7 @@ def _loadMacho(vw, filebytes, filename=None, baseaddr=None):
 
     return fname
 
+
 def parseMemory(vw, memobj, baseaddr):
     # TODO: implement
     pass
