@@ -5,6 +5,7 @@ import vstruct.qt as vs_qt
 import envi.expression as e_expr
 import envi.qt.config as e_q_config
 
+import vqt.cli as vq_cli
 import vqt.main as vq_main
 import vqt.colors as vq_colors
 import vqt.qpython as vq_python
@@ -201,6 +202,107 @@ class VQVivMainWindow(viv_base.VivEventDist, vq_app.VQMainCmdWindow):
         if ok:
             self.vw.setVaSetRow('Bookmarks', (va, str(bname)))
 
+    def getMemoryWidgets(self):
+        return self.views.get('VQVivMemoryView', [])
+
+    def getMemWidgetsByName(self, name='viv', firstonly=True):
+        '''
+        Returns a list of Memory View Widgets with the given name.
+        If "firstonly" is True, only return the first one or None(not a list)
+
+        Returns a tuple of (Widget, DockWidget).  The "Widget" is obtained from
+        the DockWidget, but they both have different powers.
+        '''
+        logger.debug("getWindowsByName(%r, firstonly=%r)", name, firstonly)
+        out = []
+
+        for vqDW in self.getMemoryWidgets():
+            w = vqDW.widget()
+            if w.getEnviNavName() == name:
+                if firstonly:
+                    return w, vqDW
+
+                out.append((w,vqDW))
+
+        if firstonly:   # if firstonly and we don't have one, return None
+            return None
+
+        return out
+
+    def getFuncGraphs(self):
+        return self.views.get('VQVivFuncgraphView', [])
+
+    def getFuncGraphsByName(self, name='FuncGraph0', firstonly=True):
+        '''
+        Returns a list of Dock Widgets which have a "getEnviNavName"
+        This includes MemoryViews and FuncGraphs
+        '''
+        logger.debug("getFuncGraphsByName()")
+
+        out = []
+        for vqDW in self.getFuncGraphs():
+            w = vqDW.widget()
+            if firstonly:
+                return w, vqDW
+
+            out.append((w, vqDW))
+
+        if firstonly:   # if firstonly and we don't have one, return None
+            return None
+
+        return out
+
+    def sendMemWidgetTo(self, va, wname='viv', firstonly=False):
+        '''
+        Tells the named Envi Nav Widget to navigate to the given VA
+        '''
+        logger.debug("sendMemWidgetsTo(0x%x, wname=%r)", va, wname)
+        for win in self.getMemWidgetsByName(wname, firstonly=False):
+            w, vqFW = win
+
+            logger.debug("sending %r to %r", w, hex(va))
+            w.enviNavGoto(hex(va))
+            if firstonly:
+                break
+        return True
+
+    def sendFuncGraphTo(self, va, wname='funcgraph0', firstonly=False):
+        '''
+        Tells the named Envi Nav Widget to navigate to the given VA
+        '''
+        logger.debug("sendFuncGraphTo(0x%x, wname=%r)", va, wname)
+        for win in self.getFuncGraphsByName(wname, firstonly=False):
+            w, vqFW = win
+
+            logger.debug("sending %r to %r", w, hex(va))
+            w.enviNavGoto(hex(va))
+            if firstonly:
+                break
+        return True
+
+    def getCliBar(self):
+        '''
+        Returns the CLI Bar object
+        '''
+        for c in self.children():
+            if isinstance(c, vq_cli.VQCli):
+                return c
+
+    def getCliText(self):
+        '''
+        Get the text from the GUI's CLI Bar (at the bottom)
+        '''
+        cli = self.getCliBar()
+        return cli.input.text()
+
+    def setCliText(self, text):
+        '''
+        Set the text in the GUI's CLI Bar (at the bottom)
+        '''
+        logger.debug("setCliText(%r)" % text)
+        cli = self.getCliBar()
+        cli.input.setText(text)
+
     def _menuEditPrefs(self):
         configs = []
         configs.append(('Vivisect', self.vw.config.viv))
@@ -353,12 +455,6 @@ class VQVivMainWindow(viv_base.VivEventDist, vq_app.VQMainCmdWindow):
     def _menuToolsDebug(self):
         viv_vdbext.runVdb(self)
 
-    def _menuViewFuncGraph(self):
-        self.vqBuildDockWidget('VQVivFuncgraphView', area=QtCore.Qt.TopDockWidgetArea)
-
-    def _menuViewSymboliks(self):
-        self.vqBuildDockWidget('VivSymbolikFuncPane', area=QtCore.Qt.TopDockWidgetArea)
-
     def _menuFileOpen(self):
         # TODO: Add something to change the workspace storage name,
         # and also to list the currently loaded files
@@ -435,28 +531,73 @@ class VQVivMainWindow(viv_base.VivEventDist, vq_app.VQMainCmdWindow):
         self.vqBuildDockWidget('VQPythonView', area=QtCore.Qt.RightDockWidgetArea)
 
     def _menuViewStrings(self):
-        self.vqBuildDockWidget('VQVivStringsView', area=QtCore.Qt.RightDockWidgetArea)
-
+        self.newStringsView()
     def _menuViewStructs(self):
-        self.vqBuildDockWidget('VQVivStructsView', area=QtCore.Qt.RightDockWidgetArea)
-
+        self.newStructsView()
     def _menuViewSegments(self):
-        self.vqBuildDockWidget('VQVivSegmentsView', area=QtCore.Qt.RightDockWidgetArea)
-
+        self.newSegmentsView()
     def _menuViewImports(self):
-        self.vqBuildDockWidget('VQVivImportsView', area=QtCore.Qt.RightDockWidgetArea)
-
+        self.newImportsView()
     def _menuViewExports(self):
-        self.vqBuildDockWidget('VQVivExportsView', area=QtCore.Qt.RightDockWidgetArea)
-
+        self.newExportsView()
     def _menuViewFunctions(self):
-        self.vqBuildDockWidget('VQVivFunctionsView', area=QtCore.Qt.RightDockWidgetArea)
-
+        self.newFunctionsView()
     def _menuViewNames(self):
-        self.vqBuildDockWidget('VQVivNamesView', area=QtCore.Qt.RightDockWidgetArea)
-
+        self.newNamesView()
     def _menuViewMemory(self):
-        self.vqBuildDockWidget('VQVivMemoryView', area=QtCore.Qt.TopDockWidgetArea)
+        self.newMemoryView()
+    def _menuViewFuncGraph(self):
+        self.newFuncGraphView()
+    def _menuViewSymboliks(self):
+        self.newSymbolikFuncView()
+
+    @idlethread
+    def newPythonView(self, floating=False):
+        self.vqBuildDockWidget('VQPythonView', floating=floating, area=QtCore.Qt.RightDockWidgetArea)
+
+    @idlethread
+    def newStringsView(self, floating=False):
+        self.vqBuildDockWidget('VQVivStringsView', floating=floating, area=QtCore.Qt.RightDockWidgetArea)
+
+    @idlethread
+    def newStructsView(self, floating=False):
+        self.vqBuildDockWidget('VQVivStructsView', floating=floating, area=QtCore.Qt.RightDockWidgetArea)
+
+    @idlethread
+    def newSegmentsView(self, floating=False):
+        self.vqBuildDockWidget('VQVivSegmentsView', floating=floating, area=QtCore.Qt.RightDockWidgetArea)
+
+    @idlethread
+    def newImportsView(self, floating=False):
+        self.vqBuildDockWidget('VQVivImportsView', floating=floating, area=QtCore.Qt.RightDockWidgetArea)
+
+    @idlethread
+    def newExportsView(self, floating=False):
+        self.vqBuildDockWidget('VQVivExportsView', floating=floating, area=QtCore.Qt.RightDockWidgetArea)
+
+    @idlethread
+    def newFunctionsView(self, floating=False):
+        self.vqBuildDockWidget('VQVivFunctionsView', floating=floating, area=QtCore.Qt.RightDockWidgetArea)
+
+    @idlethread
+    def newNamesView(self, floating=False):
+        self.vqBuildDockWidget('VQVivNamesView', floating=floating, area=QtCore.Qt.RightDockWidgetArea)
+
+    @idlethread
+    def newMemoryView(self, name='viv', floating=False):
+        dock, widget = self.vqBuildDockWidget('VQVivMemoryView', floating=floating, area=QtCore.Qt.TopDockWidgetArea)
+        widget.setMemWindowName(name)
+
+    @idlethread
+    def newFuncGraphView(self, name=None, floating=False):
+        dock, widget = self.vqBuildDockWidget('VQVivFuncgraphView', floating=floating, area=QtCore.Qt.TopDockWidgetArea)
+        if name is not None:
+            widget.setMemWindowName(name)
+
+    @idlethread
+    def newSymbolikFuncView(self, floating=False):
+        self.vqBuildDockWidget('VivSymbolikFuncPane', floating=floating, area=QtCore.Qt.TopDockWidgetArea)
+
 
     def _menuWindowFullscreen(self):
         if not self.windowState & QtCore.Qt.WindowFullScreen:
