@@ -2,7 +2,7 @@ import os
 import unittest
 import functools
 
-import vivisect
+import vivisect.cli as v_cli
 
 
 class MockVw(object):
@@ -37,7 +37,7 @@ def getTestWorkspace(*paths):
     if not testdir:
         raise unittest.SkipTest('VIVTESTFILES env var not found!')
     fpath = os.path.join(testdir, *paths)
-    vw = vivisect.VivWorkspace()
+    vw = v_cli.VivCli()
     vw.loadFromFile(fpath)
     vw.analyze()
     return vw
