@@ -3,6 +3,7 @@ import logging
 import unittest
 
 import envi
+import envi.memory as e_memory
 
 import vivisect
 import vivisect.exc as v_exc
@@ -45,6 +46,35 @@ class VivisectTest(unittest.TestCase):
                 self.assertEqual((xflags, isint(xflags)),
                                  (xflags, True))
 
+    def test_cli_search(self):
+        '''
+        Test that EnviCli.do_search works
+        '''
+        #TODO: make real tests with asserts
+        self.chgrp_vw.do_search("-e utf-16le foo")
+        self.chgrp_vw.do_search("-e utf-16le foo")
+        self.chgrp_vw.do_search("-X 41414141")
+        self.chgrp_vw.do_search("-E 0x41414141")
+        self.chgrp_vw.do_search("-E 0x41414142")
+        self.chgrp_vw.do_search("-r 0x4141.*42")
+        self.chgrp_vw.do_search("-r 0x4141.*42 -c")
+        self.chgrp_vw.do_search("-c -r qsort")
+        self.chgrp_vw.do_search("-c -r qsort -R 0x8048000:0x200")
+        self.chgrp_vw.do_search("-c -r qsort -R 0x8048000:0x2000")
+
+    def test_cli_searchopcode(self):
+        '''
+        Test that VivCli.do_searchopcodes works
+        '''
+        #TODO: make real tests with asserts
+        self.chgrp_vw.do_searchopcodes('foo')
+        self.chgrp_vw.do_searchopcodes('-f 0x08050200 ret')
+        self.chgrp_vw.do_searchopcodes('-c rol')
+        self.chgrp_vw.do_searchopcodes('-o rol')
+        self.chgrp_vw.do_searchopcodes('-t rol')
+        self.chgrp_vw.do_searchopcodes('-M red rol')
+        self.chgrp_vw.do_searchopcodes('-f 0x08050200 -R r.t')
+
     def test_loc_types(self):
         '''
         Test that we have data consistency in locations
@@ -61,6 +91,9 @@ class VivisectTest(unittest.TestCase):
                     self.assertTrue(type(linfo) in (int, str, list))
 
     def test_vaset_populate(self):
+        '''
+        Make sure the the VASEts are populated in roughly the way we expect
+        '''
         vw = self.vdir_vw
         ans = {
             'FileSymbols': [('', 0), ('filenamecat-lgpl.c', 0), ('gettime.c', 0),
@@ -185,10 +218,10 @@ class VivisectTest(unittest.TestCase):
         self.assertTrue(len(vw.getLocations()) > 76000)
 
         # tuples are Name, Number of Locations, Size in bytes, Percentage of space
-        ans = {0: ('Undefined', 0, 53840, 14),
-               1: ('Num/Int',   727, 3922, 1),
-               2: ('String',    256, 6448, 1),
-               3: ('Unicode',   172, 5530, 1),
+        ans = {0: ('Undefined', 0, 53924, 14),
+               1: ('Num/Int',   715, 3738, 0),
+               2: ('String',    265, 6485, 1),
+               3: ('Unicode',   174, 5593, 1),
                4: ('Pointer',   360, 2880, 0),
                5: ('Opcode',    72565, 279449, 74),
                6: ('Structure', 1009, 12380, 3),
@@ -776,3 +809,27 @@ class VivisectTest(unittest.TestCase):
                 cb = vw.getCodeBlock(nid)
                 self.assertEqual(nid, cb[0])
                 self.assertEqual(fva, cb[2])
+
+    def test_firefox_segments(self):
+        vw = self.firefox_vw
+        ans = {
+            'PE_Header': (0x140000000, 0x1000, e_memory.MM_READ),
+            '.text': (0x140001000, 0x48f80, e_memory.MM_READ | e_memory.MM_EXEC),
+            '.rdata': (0x14004a000, 0xbf7c, e_memory.MM_READ),
+            '.data': (0x140056000, 0x2998, e_memory.MM_READ | e_memory.MM_WRITE),
+            '.pdata': (0x140059000, 0x2f28, e_memory.MM_READ),
+            '.00cfg': (0x14005c000, 0x10, e_memory.MM_READ),
+            '.freestd': (0x14005d000, 0x10, e_memory.MM_READ),
+            '.tls': (0x14005e000, 0x11, e_memory.MM_READ | e_memory.MM_WRITE),
+            '.reloc': (0x140092000, 0x338, e_memory.MM_READ),
+        }
+        for sva, ssize, sname, sfname in vw.getSegments():
+            self.assertEqual(ans[sname][0], sva)
+            self.assertEqual(ans[sname][1], ssize)
+            self.assertEqual(sfname, 'firefox')
+
+            mva, msize, flags, mfname = vw.getMemoryMap(sva)
+            self.assertEqual(mva, sva)
+            self.assertEqual(msize, ssize)
+            self.assertEqual(flags, ans[sname][2])
+            self.assertEqual(mfname, sfname)
