@@ -1,6 +1,7 @@
 import struct
 import binascii
 
+
 class v_enum(object):
     def __init__(self):
         object.__setattr__(self, '_vs_reverseMap', {})
@@ -14,6 +15,7 @@ class v_enum(object):
         '''Maps an integer to its name. Returns default if not found'''
         return self._vs_reverseMap.get(val, default)
 
+
 class v_bitmask(object):
     def __init__(self):
         object.__setattr__(self, "_vs_reverseMap", {})
@@ -119,17 +121,19 @@ class v_prim(v_base):
     def __hash__(self):
         return hash(self.vsGetValue())
 
+
 num_fmts = {
-    (True,1):'>B',
-    (True,2):'>H',
-    (True,4):'>I',
-    (True,8):'>Q',
-    (False,1):'<B',
-    (False,2):'<H',
-    (False,4):'<I',
-    (False,8):'<Q',
+    (True, 1): '>B',
+    (True, 2): '>H',
+    (True, 4): '>I',
+    (True, 8): '>Q',
+    (False, 1): '<B',
+    (False, 2): '<H',
+    (False, 4): '<I',
+    (False, 8): '<Q',
 }
 
+
 class v_number(v_prim):
     _vs_length = 1
 
@@ -139,10 +143,10 @@ class v_number(v_prim):
         self._vs_value = value
         self._vs_enum = enum
         self._vs_length = self.__class__._vs_length
-        self._vs_fmt = num_fmts.get( (bigend, self._vs_length) )
+        self._vs_fmt = num_fmts.get((bigend, self._vs_length))
 
         # TODO: could use envi.bits, but do we really want to dep on envi?
-        self.maxval = (2**(8 * self._vs_length)) - 1
+        self.maxval = (2 ** (8 * self._vs_length)) - 1
 
     def vsGetValue(self):
         return self._vs_value
@@ -161,13 +165,13 @@ class v_number(v_prim):
         sizeoff = offset + self._vs_length
 
         if self._vs_fmt is not None:
-            b = fbytes[ offset : sizeoff ]
+            b = fbytes[offset:sizeoff]
             self._vs_value = struct.unpack(self._vs_fmt, b)[0]
 
         else:
             r = []
             for i in range(self._vs_length):
-                r.append( ord( fbytes[ offset + i ] ) )
+                r.append(ord(fbytes[offset + i]))
 
             if not self._vs_bigend:
                 r.reverse()
@@ -187,22 +191,19 @@ class v_number(v_prim):
 
         r = []
         for i in range(self._vs_length):
-            r.append( chr( (self._vs_value >> (i*8)) & 0xff) )
+            r.append((self._vs_value >> (i*8)) & 0xff)
 
         if self._vs_bigend:
             r.reverse()
 
-        return ''.join(r)
+        return bytes(r)
 
     def vsSetValue(self, value):
-        self._vs_value = long(value & self.maxval)
+        self._vs_value = value & self.maxval
 
     def __int__(self):
         return int(self._vs_value)
 
-    def __long__(self):
-        return long(self._vs_value)
-
     def __str__(self):
         v = self.vsGetValue()
         if self._vs_enum is not None:
@@ -211,68 +212,118 @@ class v_number(v_prim):
 
     ##################################################################
     # Implement the number API
+    def __add__(self, other):
+        return int(self) + int(other)
+    def __sub__(self, other):
+        return int(self) - int(other)
+    def __mul__(self, other):
+        return int(self) * int(other)
+    def __div__(self, other):
+        return int(self) / int(other)
+    def __floordiv__(self, other):
+        return int(self) // int(other)
 
-    def __add__(self, other): return long(self) + long(other)
-    def __sub__(self, other): return long(self) - long(other)
-    def __mul__(self, other): return long(self) * long(other)
-    def __div__(self, other): return long(self) / long(other)
-    def __floordiv__(self, other): return long(self) // long(other)
-    def __mod__(self, other): return long(self) % long(other)
-    def __divmod__(self, other): return divmod(long(self), long(other))
-    def __pow__(self, other, modulo=None): return pow(long(self), long(other), modulo)
-    def __lshift__(self, other): return long(self) << long(other)
-    def __rshift__(self, other): return long(self) >> long(other)
-    def __and__(self, other): return long(self) & long(other)
-    def __xor__(self, other): return long(self) ^ long(other)
-    def __or__(self, other): return long(self) | long(other)
+    def __mod__(self, other):
+        return int(self) % int(other)
+    def __divmod__(self, other):
+        return divmod(int(self), int(other))
+    def __pow__(self, other, modulo=None):
+        return pow(int(self), int(other), modulo)
+    def __lshift__(self, other):
+        return int(self) << int(other)
+    def __rshift__(self, other):
+        return int(self) >> int(other)
+    def __and__(self, other):
+        return int(self) & int(other)
+    def __xor__(self, other):
+        return int(self) ^ int(other)
+    def __or__(self, other):
+        return int(self) | int(other)
+
+    def __ge__(self, other):
+        return int(self) >= int(other)
+    def __le__(self, other):
+        return int(self) <= int(other)
+    def __gt__(self, other):
+        return int(self) > int(other)
+    def __lt__(self, other):
+        return int(self) < int(other)
+    def __eq__(self, other):
+        return int(self) == int(other)
 
     # Operator swapped variants
-    def __radd__(self, other): return long(other) + long(self)
-    def __rsub__(self, other): return long(other) - long(self)
-    def __rmul__(self, other): return long(other) * long(self)
-    def __rdiv__(self, other): return long(other) / long(self)
-    def __rfloordiv__(self, other): return long(other) // long(self)
-    def __rmod__(self, other): return long(other) % long(self)
-    def __rdivmod__(self, other): return divmod(long(other), long(self))
-    def __rpow__(self, other, modulo=None): return pow(long(other), long(self), modulo)
-    def __rlshift__(self, other): return long(other) << long(self)
-    def __rrshift__(self, other): return long(other) >> long(self)
-    def __rand__(self, other): return long(other) & long(self)
-    def __rxor__(self, other): return long(other) ^ long(self)
-    def __ror__(self, other): return long(other) | long(self)
+    def __radd__(self, other): return int(other) + int(self)
+    def __rsub__(self, other): return int(other) - int(self)
+    def __rmul__(self, other): return int(other) * int(self)
+    def __rdiv__(self, other): return int(other) / int(self)
+    def __rfloordiv__(self, other): return int(other) // int(self)
+    def __rmod__(self, other): return int(other) % int(self)
+    def __rdivmod__(self, other): return divmod(int(other), int(self))
+    def __rpow__(self, other, modulo=None): return pow(int(other), int(self), modulo)
+    def __rlshift__(self, other): return int(other) << int(self)
+    def __rrshift__(self, other): return int(other) >> int(self)
+    def __rand__(self, other): return int(other) & int(self)
+    def __rxor__(self, other): return int(other) ^ int(self)
+    def __ror__(self, other): return int(other) | int(self)
 
     # Inplace variants
-    def __iadd__(self, other): self.vsSetValue(self+other); return self
-    def __isub__(self, other): self.vsSetValue(self - other); return self
-    def __imul__(self, other): self.vsSetValue(self*other); return self
-    def __idiv__(self, other): self.vsSetValue(self/other); return self
-    def __ifloordiv__(self, other): self.vsSetValue(self // other); return self
-    def __imod__(self, other): self.vsSetValue(self % other); return self
-    def __ipow__(self, other, modulo=None): self.vsSetValue(pow(self, other, modulo)); return self
-    def __ilshift__(self, other): self.vsSetValue(self << other); return self
-    def __irshift__(self, other): self.vsSetValue(self >> other); return self
-    def __iand__(self, other): self.vsSetValue(self & other); return self
-    def __ixor__(self, other): self.vsSetValue(self ^ other); return self
-    def __ior__(self, other): self.vsSetValue(self | other); return self
+    def __iadd__(self, other):
+        self.vsSetValue(self+other)
+        return self
+    def __isub__(self, other):
+        self.vsSetValue(self - other)
+        return self
+    def __imul__(self, other):
+        self.vsSetValue(self*other)
+        return self
+    def __idiv__(self, other):
+        self.vsSetValue(self/other)
+        return self
+    def __ifloordiv__(self, other):
+        self.vsSetValue(self // other)
+        return self
+    def __imod__(self, other):
+        self.vsSetValue(self % other)
+        return self
+    def __ipow__(self, other, modulo=None):
+        self.vsSetValue(pow(self, other, modulo))
+        return self
+    def __ilshift__(self, other):
+        self.vsSetValue(self << other)
+        return self
+    def __irshift__(self, other):
+        self.vsSetValue(self >> other)
+        return self
+    def __iand__(self, other):
+        self.vsSetValue(self & other)
+        return self
+    def __ixor__(self, other):
+        self.vsSetValue(self ^ other)
+        return self
+    def __ior__(self, other):
+        self.vsSetValue(self | other)
+        return self
 
     # operator helpers
-    def __neg__(self): return -(long(self))
-    def __pos__(self): return +(long(self))
-    def __abs__(self): return abs(long(self))
-    def __invert__(self): return ~(long(self))
+    def __neg__(self):
+        return -(int(self))
+    def __pos__(self):
+        return +(int(self))
+    def __abs__(self):
+        return abs(int(self))
+    def __invert__(self):
+        return ~(int(self))
 
     # index use helper
-    def __index__(self): return long(self)
-
-    def __coerce__(self, other):
-        try:
-            return long(self),long(other)
-        except Exception:
-            return NotImplemented
+    def __index__(self):
+        return int(self)
 
     # Print helpers
-    def __hex__(self): return hex(long(self))
-    def __oct__(self): return oct(long(self))
+    def __hex__(self):
+        return hex(int(self))
+    def __oct__(self):
+        return oct(int(self))
+
 
 class v_snumber(v_number):
     _vs_length = 1
@@ -289,7 +340,7 @@ class v_snumber(v_number):
         if value & self.smask:
             value = value - self.maxval - 1
 
-        self._vs_value = long(value)
+        self._vs_value = int(value)  # TODO
 
 class v_uint8(v_number):
     _vs_builder = True
@@ -352,10 +403,10 @@ class v_ptr64(v_ptr):
     _vs_length = 8
 
 float_fmts = {
-    (True,4):'>f',
-    (True,8):'>d',
-    (False,4):'<f',
-    (False,8):'<d',
+    (True,4): '>f',
+    (True,8): '>d',
+    (False,4): '<f',
+    (False,8): '<d',
 }
 
 class v_float(v_prim):
@@ -415,42 +466,41 @@ class v_float(v_prim):
         if self._vs_bigend:
             r.reverse()
 
-        return ''.join(r)
+        return b''.join(r)
 
     def __int__(self):
         return int(self._vs_value)
+    def __float__(self):
+        return float(self._vs_value)
 
-    def __long__(self):
-        return long(self._vs_value)
-
-    def __add__(self, other): return double(self) + double(other)
-    def __sub__(self, other): return double(self) - double(other)
-    def __mul__(self, other): return double(self) * double(other)
-    def __div__(self, other): return double(self) / double(other)
-    def __floordiv__(self, other): return double(self) // double(other)
-    def __mod__(self, other): return double(self) % double(other)
-    def __divmod__(self, other): return divmod(double(self), double(other))
-    def __pow__(self, other, modulo=None): return pow(double(self), double(other), modulo)
-    def __lshift__(self, other): return double(self) << double(other)
-    def __rshift__(self, other): return double(self) >> double(other)
-    def __and__(self, other): return double(self) & double(other)
-    def __xor__(self, other): return double(self) ^ double(other)
-    def __or__(self, other): return double(self) | double(other)
+    def __add__(self, other): return float(self) + float(other)
+    def __sub__(self, other): return float(self) - float(other)
+    def __mul__(self, other): return float(self) * float(other)
+    def __div__(self, other): return float(self) / float(other)
+    def __floordiv__(self, other): return float(self) // float(other)
+    def __mod__(self, other): return float(self) % float(other)
+    def __divmod__(self, other): return divmod(float(self), float(other))
+    def __pow__(self, other, modulo=None): return pow(float(self), float(other), modulo)
+    def __lshift__(self, other): return float(self) << float(other)
+    def __rshift__(self, other): return float(self) >> float(other)
+    def __and__(self, other): return float(self) & float(other)
+    def __xor__(self, other): return float(self) ^ float(other)
+    def __or__(self, other): return float(self) | float(other)
 
     # Operator swapped variants
-    def __radd__(self, other): return double(other) + double(self)
-    def __rsub__(self, other): return double(other) - double(self)
-    def __rmul__(self, other): return double(other) * double(self)
-    def __rdiv__(self, other): return double(other) / double(self)
-    def __rfloordiv__(self, other): return double(other) // double(self)
-    def __rmod__(self, other): return double(other) % double(self)
-    def __rdivmod__(self, other): return divmod(double(other), double(self))
-    def __rpow__(self, other, modulo=None): return pow(double(other), double(self), modulo)
-    def __rlshift__(self, other): return double(other) << double(self)
-    def __rrshift__(self, other): return double(other) >> double(self)
-    def __rand__(self, other): return double(other) & double(self)
-    def __rxor__(self, other): return double(other) ^ double(self)
-    def __ror__(self, other): return double(other) | double(self)
+    def __radd__(self, other): return float(other) + float(self)
+    def __rsub__(self, other): return float(other) - float(self)
+    def __rmul__(self, other): return float(other) * float(self)
+    def __rdiv__(self, other): return float(other) / float(self)
+    def __rfloordiv__(self, other): return float(other) // float(self)
+    def __rmod__(self, other): return float(other) % float(self)
+    def __rdivmod__(self, other): return divmod(float(other), float(self))
+    def __rpow__(self, other, modulo=None): return pow(float(other), float(self), modulo)
+    def __rlshift__(self, other): return float(other) << float(self)
+    def __rrshift__(self, other): return float(other) >> float(self)
+    def __rand__(self, other): return float(other) & float(self)
+    def __rxor__(self, other): return float(other) ^ float(self)
+    def __ror__(self, other): return float(other) | float(self)
 
     # Inplace variants
     def __iadd__(self, other): self.vsSetValue(self+other); return self
@@ -467,23 +517,17 @@ class v_float(v_prim):
     def __ior__(self, other): self.vsSetValue(self | other); return self
 
     # operator helpers
-    def __neg__(self): return -(double(self))
-    def __pos__(self): return +(double(self))
-    def __abs__(self): return abs(double(self))
-    def __invert__(self): return ~(double(self))
+    def __neg__(self): return -(float(self))
+    def __pos__(self): return +(float(self))
+    def __abs__(self): return abs(float(self))
+    def __invert__(self): return ~(float(self))
 
     # index use helper
-    def __index__(self): return double(self)
-
-    def __coerce__(self, other):
-        try:
-            return double(self),double(other)
-        except Exception:
-            return NotImplemented
+    def __index__(self): return float(self)
 
     # Print helpers
-    def __hex__(self): return hex(double(self))
-    def __oct__(self): return oct(double(self))
+    def __hex__(self): return hex(float(self))
+    def __oct__(self): return oct(float(self))
 
 class v_double(v_float):
     _vs_length = 8
@@ -499,7 +543,7 @@ class v_bytes(v_prim):
     def __init__(self, size=0, vbytes=None):
         v_prim.__init__(self)
         if vbytes is None:
-            vbytes = '\x00' * size
+            vbytes = b'\x00' * size
         self._vs_length = len(vbytes)
         self._vs_value = vbytes
         self._vs_align = 1
@@ -524,10 +568,10 @@ class v_bytes(v_prim):
         self._vs_fmt = '%ds' % size
         # Either chop or expand my string...
         b = self._vs_value[:size]
-        self._vs_value = b.ljust(size, '\x00')
+        self._vs_value = b.ljust(size, b'\x00')
 
     def __repr__(self):
-        return binascii.hexlify(self._vs_value)
+        return binascii.hexlify(self._vs_value).decode('utf-8')
 
 class v_str(v_prim):
     '''
@@ -538,11 +582,11 @@ class v_str(v_prim):
 
     _vs_builder = True
 
-    def __init__(self, size=4, val=''):
+    def __init__(self, size=4, val=b''):
         v_prim.__init__(self)
         self._vs_length = size
         self._vs_fmt = '%ds' % size
-        self._vs_value = val.ljust(size, '\x00')
+        self._vs_value = val.ljust(size, b'\x00')
         self._vs_align = 1
 
     def vsParse(self, fbytes, offset=0):
@@ -554,11 +598,13 @@ class v_str(v_prim):
         return self._vs_value
 
     def vsGetValue(self):
-        s = self._vs_value.split("\x00")[0]
+        s = self._vs_value.split(b'\x00')[0].decode('utf-8')
         return s
 
     def vsSetValue(self, val):
-        self._vs_value = val.ljust(self._vs_length, '\x00')
+        if type(val) is str:
+            val = val.encode('utf-8')
+        self._vs_value = val.ljust(self._vs_length, b'\x00')
 
     def vsSetLength(self, size):
         size = int(size)
@@ -566,7 +612,7 @@ class v_str(v_prim):
         self._vs_fmt = '%ds' % size
         # Either chop or expand my string...
         b = self._vs_value[:size]
-        self._vs_value = b.ljust(size, '\x00')
+        self._vs_value = b.ljust(size, b'\x00')
 
 class v_zstr(v_prim):
     '''
@@ -576,13 +622,13 @@ class v_zstr(v_prim):
 
     _vs_builder = True
 
-    def __init__(self, val='', align=1):
+    def __init__(self, val=b'', align=1):
         v_prim.__init__(self)
         self._vs_align = align
-        self.vsParse(val+'\x00')
+        self.vsParse(val + b'\x00')
 
     def vsParse(self, fbytes, offset=0):
-        nulloff = fbytes.find('\x00', offset)
+        nulloff = fbytes.find(b'\x00', offset)
         if nulloff == -1:
             raise Exception('v_zstr found no NULL terminator!')
 
@@ -596,8 +642,8 @@ class v_zstr(v_prim):
         return nulloff
 
     def vsParseFd(self, fd):
-        ret = ''
-        while not ret.endswith('\x00'):
+        ret = b''
+        while not ret.endswith(b'\x00'):
             y = fd.read(1)
             if not y:
                 raise Exception('v_zstr file ended before NULL')
@@ -608,13 +654,13 @@ class v_zstr(v_prim):
         return self._vs_value
 
     def vsGetValue(self):
-        return self._vs_value[:-self._vs_align_pad]
+        return self._vs_value[:-self._vs_align_pad].decode('utf-8')
 
     def vsSetValue(self, val):
         # FIXME: just call vsParse?
         length = len(val)
         diff = self._vs_align - (length % self._vs_align)
-        self._vs_value = val + '\x00'*(diff)
+        self._vs_value = val + b'\x00'*(diff)
         self._vs_length = len(self._vs_value)
         self._vs_align_pad = diff
 
@@ -630,9 +676,9 @@ class v_wstr(v_str):
 
     _vs_builder = True
 
-    def __init__(self, size=4, encode='utf-16le', val=''):
+    def __init__(self, size=4, encode='utf-16le', val=b''):
         v_str.__init__(self)
-        b = val.ljust(size, '\x00').encode(encode)
+        b = val.ljust(size, b'\x00')
         self._vs_length = len(b)
         self._vs_value = b
         self._vs_encode = encode
@@ -648,7 +694,7 @@ class v_wstr(v_str):
 
     def vsSetValue(self, val):
         rbytes = val.encode(self._vs_encode)
-        self._vs_value = rbytes.ljust(len(self), '\x00')
+        self._vs_value = rbytes.ljust(len(self), b'\x00')
 
     def vsGetValue(self):
         s = self._vs_value.decode(self._vs_encode, errors='replace')
@@ -661,16 +707,16 @@ class v_zwstr(v_str):
 
     _vs_builder = True
 
-    def __init__(self, encode='utf-16le', val='', align=2):
+    def __init__(self, encode='utf-16le', val=b'', align=2):
         v_str.__init__(self)
         self._vs_encode = encode
         self._vs_align = align
-        self.vsParse(val+'\x00\x00')
+        self.vsParse(val + b'\x00\x00')
 
     def vsParse(self, fbytes, offset=0):
         nulloff = offset
         while nulloff < len(fbytes):
-            nulloff = fbytes.find('\x00\x00', nulloff)
+            nulloff = fbytes.find(b'\x00\x00', nulloff)
             if nulloff == -1:
                 raise Exception('v_wzstr found no NULL terminator!')
             #make sure that this is acutally the null word by checking if
@@ -709,8 +755,8 @@ class v_zwstr(v_str):
         rbytes = val.encode(self._vs_encode)
         length = len(rbytes)
         diff = self._vs_align - (length % self._vs_align)
-        self._vs_value = rbytes + '\x00'*(diff)
-        self._vs_value = rbytes.ljust(len(self), '\x00')
+        self._vs_value = rbytes + b'\x00' * (diff)
+        self._vs_value = rbytes.ljust(len(self), b'\x00')
         self._vs_length = len(self._vs_value)
         self._vs_align_pad = diff
 
@@ -728,9 +774,9 @@ class GUID(v_prim):
         """
         v_prim.__init__(self)
         self._vs_length = 16
-        self._vs_value = "\x00" * 16
-        self._vs_fmt = "16s"
-        self._guid_fields = (0,0,0,0,0,0,0,0,0,0,0)
+        self._vs_value = '\x00' * 16
+        self._vs_fmt = '16s'
+        self._guid_fields = (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
         if guidstr is not None:
             self._parseGuidStr(guidstr)
 
@@ -743,12 +789,12 @@ class GUID(v_prim):
         return struct.pack("<IHH8B", *self._guid_fields)
 
     def _parseGuidStr(self, gstr):
-        gstr = gstr.replace("{","")
-        gstr = gstr.replace("}","")
-        gstr = gstr.replace("-","")
+        gstr = gstr.replace('{', '')
+        gstr = gstr.replace('}', '')
+        gstr = gstr.replace('-', '')
         bytes = binascii.unhexlify(gstr)
         # Totally cheating... ;)
-        self._guid_fields = struct.unpack(">IHH8B", bytes)
+        self._guid_fields = struct.unpack('>IHH8B', bytes)
 
     def vsSetValue(self, guidstr):
         self._parseGuidStr(guidstr)
